/*
 * @(#)dbArbiterConnection.java	1.0 98/07/07
 * 
 * Copyright (c) 1998 Studio Associato I-Tech All Rights Reserved.
 * 
 * 
 * I-Tech MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. I-Tech SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.0
 * 
 */
package com.studioitech.gulli.MJDBC;

import java.util.*;
import java.sql.*;


/**
 * <P>A Connection represents a session with a specific
 * database. Within the context of a Connection, SQL statements are
 * executed and results are returned.
 *
 * <P>This package allows user to init a lot of JDBC Connections to 
 * a database. These connections are never closed until the end of
 * the user program and they are handled in mutex way: i mean, they 
 * are multi-thread safe.
 *
 * <P>An example of benefit of this, is servlet programming. In 
 * servlet model you have an init phase (during which servlet is
 * allocated) a destroy phase (during which servlet is deallocated) 
 * and a doGet (or similiar ...). The init and destroy method are 
 * called by the HTTP server in single-thread enviroment. The doGet
 * method works in a multi-thread enviroment.
 *
 * <P>The HTTP server for every incoming GET request directed to 
 *  servlet create a new thread and execute the doGet code.
 *  
 * <P>By using this package, you can open a lot of connection to 
 * the DB during the init phase. Use them in multi-thread safe mode
 * during the doGet call. Only close them when the servlet is discarded 
 * from HTTP server.
 *
 * <P>Benefits are in term of performance. In fact, using traditional
 *  cgi-bin like paradigm, for every incoming request you have this 
 *  cycle:
 *
 * <UL>
 *		<li>Open DB
 *		<li>Perform SQL Query and get Results
 *      <li>Close DB
 * </UL>
 * 
 * Opening and Close a DB is a very costly operation. This is true in 
 * direct access to a given database via a propreatary API, This become
 * most evident if you access the database via the JDBC layer because you
 * have to count the time to load the JDBC driver. So is better to perform
 * these costly operation at the init time once.
 *
 * <P>Using this package you can move to a new paradigm, for every incoming 
 * request you have this cycle:
 * 
 * <UL> 
 *      <li>Get an available connection to the DB and mark it as used
 *      <li>Perform SQL Query and get Results
 *      <li>Mark used connection as available
 * </UL>
 * 
 * For a large volume of requests avoiding to load JDBC driver, open and 
 * close continuosly means to have a less latency in answering. 
 */

public class dbArbiterConnections
{

	private Vector arbiter;        // A vector containing all allocated connection
    private dbConnection con;      // A single connection with statement and mutex support
    private int ncons;              // number of allocated connection 

	/**
	 * Constructor of the ArbiterConnections: It opens ncons dbConnection
	 * and init their multi-thread safe handling<p> 
	 *
	 * @param JDBC_DriverName the name of JDBC driver
	 * @param url  the url to connect database (e.g.: jdbc:postgresql:database )
	 * @param usr  the name of database's user (e.g.: test)
	 * @param pwd  the password database user  (e.g.: test)
	 * @param ncons number of connection to open
     *
	 * @exception SQLException if a database-access error occurs.
	 * @exception ClassNotFoundException if the JDBC driver is not
	 *            available
	 *
	 */

	public dbArbiterConnections(String JDBC_DriverName,
								String url, String usr, 
								String pwd, int ncons)
		throws SQLException, ClassNotFoundException
	{
		try {

			// Load the Driver to access DB
			Class.forName(JDBC_DriverName);

			// Init connections
			initConnections(url, usr, pwd, ncons);

		} catch (ClassNotFoundException  cnfe)
		{
			E.debug("Problems loading JDBC driver " + cnfe.toString(), 0);
			throw cnfe;
		}
	}


	private void initConnections(String url, String usr, String pwd, int ncons)
		throws SQLException	
	{
		// Create arbiter's status
		arbiter = new Vector(ncons);
	
        // save its dimension
		this.ncons = ncons;

		// Init arbiter's status creating connection
		for (int i=0; i<ncons; i++){

				
			E.debug("Opening connection" + i, 0);		
			con = new dbConnection(url, usr, pwd);
			arbiter.addElement((Object) con);
		}

		E.debug("Arbiter initializated", 0);
	}

	/**
	 * Get an available connection in the opened pool of dbConnection<p>
	 * 
     *
	 * @return dbConnection, the chosen available connection
	 * @see dbConnection
	 */

	public dbConnection getAnAvailableCon()
	{
	
		// This is a endless cycle until we find
		// an available connection
		con = null;

		while (true)
		{
			E.debug("lookin for connection", 0);


			for (Enumeration e = arbiter.elements() ; 
				e.hasMoreElements(); ) 
			{
  
			// Fetch a connection and verify if 
			// it is available for use 

				con = ((dbConnection)e.nextElement());
				if (con.takeConnection()){

					E.debug("A connection is available", 2);
					return con;	
				}
			}
        }
	}	

	/**
	 * Close all available dbConnection <p>
	 * 
	 * @exception SQLException if a database-access error occurs
	 *
	 */
	public void shutdown()
		throws SQLException
	{

		// For all connection, close statement and db

		for (Enumeration e = arbiter.elements() ; 
			 e.hasMoreElements(); ) 
		{
  		
			con = ((dbConnection)e.nextElement());

			try {
				con.st.close();
				con.db.close();
			} catch (SQLException ex)
			{
				E.debug(ex.toString(), 0);
				throw ex;
			}
		}
	}

	/**
	 * Get the number of connection in the pool of opened dbConnection <p>
	 * 
     *
	 * @return dbConnection the number of available connections
	 * @see dbConnection
	 *
	 */


	public int getNumAllocatedCon(){

		return ncons;
	}
}

